
package controlisolation.lifecyclefsm.stateexecutor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;


public abstract class StateWork {
    
    protected static final Logger _logger =
            Logger.getLogger(StateWork.class.getName());
    private Map<String,Object> _workInputs=new HashMap<String,Object>();
    private Map<String,Object> _workOutputs=new HashMap<String,Object>();
    private List<String> _workErrors=new ArrayList<String>();
    private List<String> _workWarnings=new ArrayList<String>();
    private List<Exception> _workExceptions=new ArrayList<Exception>();
    
    private StateExecutor _stateExecutor;
    
    public abstract void doWork() throws Exception;
  
    public void addException(Exception e)
    {
       _workExceptions.add(e); 
    }
    
    public void addError(String str)
    {
      _workErrors.add(str);
    }
    
    public void addWarning(String str)
    {
      _workWarnings.add(str);
    }
    
   
    
    public boolean suscessful()
    {
        return _workErrors.isEmpty()&&
                _workExceptions.isEmpty();
    }
    
    public void setWorkInputs(Map<String,Object> workInputs)
    {
        this._workInputs=workInputs;
    }
    
    public Map<String,Object> getWorkOutputs()
    {
        return _workOutputs;
    }
    
    public List<String> getWorkErrors()
    {
      return _workErrors;
    }

    public Map<String,Object> getWorkInputs() {
        return _workInputs;
    }

  

    public List<Exception> getWorkExceptions() {
        return _workExceptions;
    }

    
    public boolean hasWarnings()
    {
        return !(this._workWarnings.isEmpty());
    }

    public List<String> getWorkWarnings() {
        return _workWarnings;
    }

    public StateExecutor getStateExecutor() {
        return _stateExecutor;
    }

    public void setStateExecutor(StateExecutor stateExecutor) {
        this._stateExecutor = stateExecutor;
    }

    public Map<String,Object> getGlobals()
    {
       return this._stateExecutor.getContainerFSM().getGlobals();
    }
    
    
    public String toString()
    {
        StringBuffer strb=new StringBuffer();
        strb.append("WORK:");
        strb.append(this.getClass().getName());
        strb.append("\n");
        
        return strb.toString();      
    }

    public static String toStringWorkInputs(StateWork w) {
        StringBuffer strb=new StringBuffer();
        strb.append("\tWORK INPUTS:\n");
        for (String key: w.getWorkInputs().keySet())
        {
            strb.append("\t\t["+key+"|");
            Object obj= w.getWorkInputs().get(key);
            strb.append(obj.toString()+"]\n");
        }
        return strb.toString();
    }
    
    public static String toStringWorkOutputs(StateWork w) {
        StringBuffer strb=new StringBuffer();
        strb.append("\tWORK OUTPUTS:\n");
        for (String key: w.getWorkOutputs().keySet())
        {
            strb.append("\t\t["+key+"|");
            Object obj= w.getWorkOutputs().get(key);
            strb.append(obj.toString()+"]\n");
        }
        return strb.toString();
    }
    
    public void pauseFor(long ms)
    {
        try {
            Thread.sleep(ms);
        } catch (InterruptedException ex) {
             _logger.log(Level.SEVERE, null, ex);
        }
    }
    
}
